## **XML, SOAP and Web Services**

There are a great number of products which implement DCOM based OPC Specifications. However, there are also some usage restrictions which have to be considered during development and use of these kind of products.

-   DCOM does not pass firewalls, i.e. direct addressing of computers through the firewall is not possible. However, this is precisely what DCOM needs to perform an internal check.
    
-   There are some devices and applications which provide or require data and which do not run on Microsoft systems. They include, for example, applications in the ERP or MDS areas as data consumers or Embedded Devices as data sources.
    

These restrictions are the reason that OPC foundation has undertaken the development of the OPC XML-DA specification. This specification is no longer based on DCOM, but on a technology independent from a specific operating system. Since this specification is explained later, the relevant components of this technology will be introduced below.

The eXtensible Markup Language (XML) is a flexible data description language which is easy to comprehend and learn. Information is exchanged by means of readable XML documents. These contain text with embedded structuring information. An XML document consists of elements and attributes. Elements can contain other elements. Attributes are assigned to elements. Structuring takes place by the selection of elements and attributes. XML defines that elements are labeled by tags which can be defined as desired. An XML document is called well-formed if it corresponds to the XML syntax; it is called valid if, in addition, it corresponds to a default schema. The schema itself is also defined using XML. Thus, a schema defines a language for structuring information in a specific area of application. The creation of XML documents and schemas as well as validation and processing of the files are supported by a variety of tools. Conversion, processing and display of the documents can be handled flexibly.

XML documents can be exchanged between applications in different ways, e.g. on diskette, by Email, using HTTP or TCP/IP. Today, the support of XML is guaranteed by practically all systems. Thus, even heterogeneous systems can easily interact by exchanging XML documents.

The Simple Object Access Protocol (SOAP) is an interaction protocol that links two technologies: XML and HTTP. HTTP is used as the transport protocol. The parameters of the interactions are described with XML. SOAP is thus predestined specifically for the Internet. SOAP is a protocol independent of object architectures (DCOM, CORBA). A SOAP telegram consists of a part describing the structure of the HTTP call (request/response, host, content type and content length). This part is included in all the HTTP telegrams. A UniversalResourceIdentifier (URI) was added, which defines the end point and the method to be called. The method parameters are transferred as XML. The programmer is responsible for mapping the SOAP protocol to a concrete implementation. In the meantime, SOAP has been submitted to the W3C (World Wide Web Consortium) for standardization. In this context the name has changed to XML protocol.
### soap.msg

```  
POST /StockQuote HTTP/1.1
Host: www.stockquoteserver.com
Content-Type: text/xml; charset="utf-8"
Content-Length: nnnn
SOAPAction: "http://www.stockquoteserver.com/QuoteCalc#GetLastTradePrice"

<SOAP-ENV:Envelope
  xmnls:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"
  SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" />
  <SOAP-ENV:Header>
      <t:Transaction
          xmlns:t="some-URI"
          SOAP-ENV:mustUnderstand="1">
              5
          </t:Transaction>
  </SOAP-ENV:Header>
  <SOAP-ENV:Body>
      <m:GetLastTradePrice xmlns:m="Some-URI">
          <symbol>DEF</symbol>
      </m:GetLastTradePrice>
  </SOAP-ENV:Body>
</SOAP-ENV:Envelope>
```

The code above shows a SOAP request sent with HTTP-Post. The first lines are part of the HTTP header. An XML document follows. The request is addressed to the host www.stockquoteserver.com. There, the method GetLastTradePrice is to be called in the program QuoteCalc.

The SOAP message has a header and a body. The header attributes describe how the request is to be processed. The body contains the method parameter; in this case, the symbol of the company whose stock price is to be queried.

Based on the technologies introduced above, it is already possible to implement distributed applications which interact via SOAP and are independent from the operating system and the hardware. However, something is still missing: a way of describing an application‘s interface and of generating program components from this description which are, on the one hand, compliant with the existing infrastructure (HTTP etc.) and which, on the other hand, can be integrated in existing programs. This is where Web Services come into play.

The programmatic interfaces made available are referred to as Web Services. SOAP is used as the interaction protocol between components.

Web Services are described using XML. The language used is WSDL (Web Services Description Language), which is standardized in the W3C. One schema defines the syntax and semantics of WSDL; based on this definition, schemas for concrete interfaces are defined. This can be done by individual manufacturers or by an organization, such as the OPC Foundation. An application interacting with the Web Service will deliver valid XML message which are compliant with the schema. The function call is sent as a XML message. This happens also with the response and a possible error information. Components that support or use Web Services can be implemented on any platform supporting XML.

The introduced technologies (XML, SOAP und Web Services) were not defined by individual companies or company groups, but by the WWW Consortium (W3C).

This fact is also of importance for future OPC specifications. In the past, the fact that OPC is only based on DCOM has been critisized. This point should go away if specifications are based on XML and Web Services.

The following table shows the current (February 2004) status of the different specifications. Recommendation stands for agreed standard, Draft stands for a standard which is still not agreed. Note stands for a rather detailed working paper.

XML Specifications | Release state | 
| :------------- | :----------: | 
| XML 1.0 | W3C Recommendation | 
| XML Schema Part 1 & 2 1.0 | W3C Recommendation | 
| SOAP/XMLP 1.2 | W3C Recommendation | 
| WSDL 2.0 | W3C Draft | 

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/blob/main/documentation_pics/CMN_Intro_002_2.jpg)

The figure shows the infrastructure used by OPC XML DA applications. The concrete Web Service is defined within the OPC XML DA Specification and described with WSDL. SOAP is used as the interaction protocol. The different systems must support the transmission and reception of calls. These parts of the software are derived from the WSDL description. TCP/IP can be used as a transport protocol under HTTP which can, in turn, run on various network protocols.
