## **OPC Data Access**

First, an "object hierarchy" is defined in the specification. The software component "OPC Data Access Server" contains an OPCServer object for every client, one or more OPCGroup objects and one or more OPCItem objects. The latter represent a data item of interest. OPCGroup objects are logical containers for OPCItem objects. The grouping can be based on different aspects (for example, the dynamics of the changed value). The hierarchy is shown in the figure.

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/CMN_Intro_003_1.jpg)

The software component OPC Client accesses the objects of the Data Access Server. There is an option of building callbacks to OPC Clients from OPCGroup objects. Data is sent from the Data Access Server to the OPC Client via this connection upon occurrence of certain conditions. Parameters are sent to the client in the callback allowing it to relate the answer to a previous method request.

By following these provisions, a standardized view of all implemented Data Access Servers from all implemented OPC Clients is guaranteed.

Another definition issue of OPC is the specification of data formats for the variables to be transferred.
![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/CMN_Intro_003_2.jpg)

These data formats contain the value of the process variable, whose type can be one of the usual Win32 data types and the data types derived from them. Furthermore, a time stamp is part of the format. It contains 8 bytes and specifies the time since 01 January 1600, with a precision of 100 ns. A client, of course, only displays the useful part of this information.

Status information on the measured value and the sensing device is transferred in the last part. The part contains 16 bits; only 8 bits are currently used. The field "Quality" describes whether the data value is "valid", "invalid" or "uncertain". Corresponding supplemental information is contained in the field "Status". In this way, it is possible to encode that a communication error exists if the value is "invalid".

The field "Limit" describes whether the value has exceeded the predetermined limit value in any direction. This limit value has no relation to the value to be transferred when an OPCGroup object is added!

Further stipulations in the Data Access Specification pertain to the description of errors and the definition of corresponding error codes. For example, there is an error code in case an OPCItem object is to be created with an invalid ItemID (OPC_E_UNKNOWNITEMID, 0xC0040007L) in the Data Access Server. The definition of errors is based on the requirements of DCOM, i.e. reserved areas for HRESULT are used. Thus, it is also possible for Data Access Server manufacturers to define their own error codes and the OPC Client can query the error string in the Data Access Server using the corresponding method (GetErrorString). This example of combining proprietary solutions with defined procedures illustrates the flexibility of OPC.

Various data acquisition methods are specified in the Data Access Specification. The functionality must be provided by every Data Access Server. Values can be read or written synchronously or asynchronously.

With synchronous data access, the method runs to completion before it returns.

In case of asynchronous reading or writing, the OPC Client is informed accordingly by the Data Access Server. Asynchronous read operations can refer to a server-internal cache (only in version 1.0A) or to the device. Values are communicated via the callback created.

A refresh call forces a callback to the connection point sink for all active items in the group.

Another manner of reading is that the OPC Client establishes a connection to an OPCGroup object of the Data Access Server (ConnectionPoint / Advise) and the latter automatically transfers data when the corresponding conditions have been established. In this case, the data is read from the device and written to the cache and the condition is calculated. The frequency of the read calls is controlled by an OPCGroup parameter.
