## **OPC Alarms and Events**

Alarm and Event Servers are used for reporting alarms and events to operator stations, logger components or management subsystems.

The underlying model describes a condition as a definable state of the Alarm and Event Server or one of its objects (for example, a process variable). Alarms are abnormal conditions. An event is a detectable occurrence of significance to the Alarm and Event Server. The type and manner in which events and conditions are defined is the responsibility of the server manufacturer and is not prescribed. However, the attributes of a condition are defined. The following event types exist:

-   condition-related events, e.g. transition to a state represented by conditions
-   tracking-related events, e.g. representing a parameter change performed by an operator
-   simple events, e.g. device failure.

The client is informed about the occurrence of an event via callbacks from OPCEventSubscription Objects. Events and conditions can be structured within organization units. Filters and acknowledgement mechanisms also exist.

Condition-related events represent a state change. States are described by conditions. Different conditions can denote different ranges of a measured value. In this case, the specification introduces multiple-state conditions. Subconditions are used to represent the substates. On the other hand, events can denote the state of the condition itself, e.g. an incoming acknowledgement can fire an event. Condition-related events can be related to OPCItem objects. The definition of conditions is a task done during configuration of the server.

Simple and tracking-related events represent changes that are not related to measured values and do not have to be acknowledged. A device failure is an example.

The figure illustrates the object hierarchy of an Alarm and Event Server.

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/CMN_Intro_004_1.jpg)

The top-level object is the OPCEventServer object. It provides methods for creating the OPCServerAreaBrowser object and the OPCEventSubscription objects.

The object OPCServerAreaBrowser provides a way for clients to browse the event area organization implemented by the server. The implementation of this object is optional.

OPCEventSubscription objects are used to manage events and to signal them to clients. OPCEventSubscription objects call methods in the advise sink on the client side.

After the OPCEventSubscription object has been added, the client can filter events.

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/CMN_Intro_004_2.jpg)

The figure shows which mandatory attribute values are returned in the event notification. The parameters present for simple events are also present for condition- and tracking-related events.

The parameter Source returns the source name of the event notification. The parameter Time contains the time of the event occurrence. The value of the parameter Type returns the event type; possible values are OPC_CONDITION_EVENT, OPC_SIMPLE_EVENT, and OPC_TRACKING_EVENT. Furthermore, the event category, the severity and a message can be sent to the client.

The following parameter is available only for tracking-related events: ActorId: the parameter denotes the client responsible for the event.

The following parameters are present only for condition-related events:

-   ConditionName: the name of the condition related to this notification.
-   SubConditionName: the name of the current subcondition related to this notification. For a single-state condition, it contains the name of this condition.
-   ChangeMask: indicates to the clients which properties of the condition have changed to have caused the event (OPC_CHANGE_ACTIVE_STATE).
-   NewState: this parameter carries the current state of the condition (OPC_CONDITION_ACTIVE, OPC_CONDITION_ENABLED, OPC_CONDITION_ACKED).
-   ConditionQuality: quality associated with the condition state. An event is also fired when the quality changes.
-   AckRequired: this flag indicates that the condition requires acknowledgment of this event.
-   ActiveTime: time that the condition became active or the time of transition into the current subcondition.
-   Cookie: server-defined cookie associated with the event notification and used by the client during acknowledgement.
-   AckId: used only for condition-related events which are acknowledgement notifications. Parameter contains the AcknowledgerId.
