## **Integration into Apache and Microsoft IIS**

An OPC XML DA Server, written with the OPC Toolkit C++ XML DA Server, can be accessed via Apache or Microsoft IIS web servers or directly without an additional web server. The integration of the OPC XML DA Server into a web server is useful, in the following cases:

-   Besides OPC XML DA, other resources need to be accessed (e.g. HTML pages) over HTTP.
-   HTTPS should be used to access any OPC XML DA-enabled web server.
    
    However, the OPC Toolkit HTTP server does not support HTTPS. By using the Apache or IIS servers, HTTPS-based access can be introduced.
    

**Apache Integration**

The Apache web server can be configured so that it forwards HTTP requests for one URL to another.

It includes the module mod_proxy. This must be configured using the configuration file, /etc/httpd/conf/httpd.conf.

In order to configure Apache to forward any requests to an OPC XML DA server, the following lines should be added to httpd.conf:

```
<IfModule mod_proxy.c>
  ProxyPass localPath forwardToURL
</IfModule>
```

If local access to the installed OPC Toolkit Demo server using the Apache URL http://apache-server/proxy/local is desired, the following should be configured:

```
<IfModule mod_proxy.c>
  ProxyPass /proxy/local http://localhost:8080/OPC/DA
</IfModule>
```

Forwarding could also be directed to another server machine. It is possible to insert more than one ProxyPass statement to the httpd.conf file to achieve this.

It may be necessary to adjust the ProxyTimeout value. This is only required if the default value of 300 is not sufficient.

In addition, if the KeepAlive Off setting is present, it should be commented out, since it setting prevents HTTP connections from being automatically closed after each request.

To activate the changed configuration, the following command should be run:

```
kill –SIGHUP `cat /var/run/httpd.pid`
```

**Microsoft IIS Integration**

For the integration into the Microsoft IIS Web server, Softing has implemented an ISAPI Extension DLL, which performs the forwarding of the HTTP requests.

In order to install the Softing IIS Proxy DLL, the following procedure should be followed:

-   Open the IIS configuration GUI (Settings -> Control Panel -> Administrative Tools -> Internet Information Services);
-   Select "Default Web Site", right-click on it and from the resulting menu, select Properties;
-   In the dialog "Default Web Site Properties", select the tab "Home Directory".

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/CPP_200_Srv_Web1.jpg)

In this dialog box, the "Execute Permissions" should be set to "Scripts only". For "Application Protection", select "Medium (Pooled)" or "Low (IIS Process)". The latter setting increases the performance of the ISAPI extension, but reduces security levels of the web server. The button "Unload" releases all loaded ISAPI extension DLLs.

Click on the "Configuration..." button. This opens the following dialog:
![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/CPP_200_Srv_Web2.jpg)

Select "Cache ISAPI applications". The Softing IIS Proxy DLL caches the connections internally to improve performance.

Clicking on the button "Add" opens the following dialog box: <br>
![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/CPP_200_Srv_Web3.jpg)

In this dialog, an association between the path to the IISProxy.dll and an arbitrary extention is created, in this case .redir. The OPC Toolkit Demo Server installation installs the IISProxy.dll to "<CommonFiles Dir>\Softing\OPC". The OPC Toolkit C++ installation installs the file to "<OPC Toolkit C++>\bin\IISProxy".

At "Verbs", you can select "All Verbs" or "Limit to:" POST, depending on the HTTP methods the IIS proxy should forward. For XML-DA OPC applications, the POST verb is sufficient. The setting "Script engine" indicates that the .redir file has to be included in a directory with "Script Execute" permissions. If "Check that file exists" is selected, it is not possible to add any file besides one with a .redir filename extension.

This concludes all required IIS configurations.

It is now necessary to create the desired redirections. Any text editor may be used to create the required .redir files in the \wwwroot directory.

A .redir file has the following syntax:

```
http://{targethost}{:port}/{pfad}  {active-timeout}  {idle-timeout}
```

The timeouts are specified in seconds and are optional. The default values of the proxy are 60 and 10.

Example: The file wwwroot\proxy\local.redir with the contents http://localhost:8080/OPC/DA

This redirects all HTTP requests from http://iis-server/proxy/local.redir to http://localhost:8080/OPC/DA.

The redirection could point to another machine. It is possible to add more than one redirection.

The access rights of the .redir files should include read access for the anonymous IIS user.
