## **DCOM Configuration**

The following configuration steps open a Windows system just far enough to allow communication via DCOM.

If these settings do not satisfy your requirements, they can be modified accordingly.

### DCOMCNFG

DCOM uses security settings to protect both the client and server from unauthorized access. The security settings can be changed with the program **DCOMCNFG**. Administrator rights must be held by the current user in order to use the program.

#### Note
```
Pure XML-DA OPC Servers do not require any DCOM Security configuration.
```
To start DCOMCNFG, select Run from the Start menu and enter DCOMCNFG. A dialog opens in which you can enter the settings listed below.

In the left panel of the DCOMCNFG (Component Services window), expand the **Console Root** node, then expand the **Component Services** node, then expand the **Computers** node and finally select Properties row from the contextual menu over the **My Computer** item.

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/DCOMCNFG_MyComputer.png)

#### Note
```
Please make sure that the DCOM security settings have not been altered (default Windows DCOM settings). If
these security settings have been modified, please adjust them so that they are a close match to the implicit ones.
```

**General DCOM properties**

Under the **Default Properties** tab of the **My Computer Properties** window, make sure that the settings are configured as follows:

|   Parameter    | Value      | 
| ------------- |:-------------:|
| Enable Distributed COM on this computer     | Checked  |
| Default Authentication Level      | Connect  |
| Default Impersonation Level     | Identify  |

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/DCOMCNFG_MyComputer_DefaultProperties.png)

Further on, under the **COM Security** tab of the **My Computer Properties** window, make sure that the settings are configured as follows:

**OPC Server COM security settings**
|   Parameter    | Value      | 
| ------------- |:-------------:|
| Access Permissions area (both Edit Limits... and Edit Default... sections)     | - Add the domain/workgroup user(s) or group of users which have remote access permissions (in order for the local OPC Servers to receive calls from remote OPC Clients)<br> - Mark both Local Access and Remote Access rights for this (these) user(s)/group(s)  |
| Launch and Activation Permissions area (both Edit Limits... and Edit Default... sections)      |- Add the domain/workgroup user(s) or group of users which have remote launch and activation permissions (in order for the local OPC Servers to be started by remote OPC Clients)<br> - Mark all of the Local Launch, Remote Launch, Local Activation and Remote Activation rights for this (these) user(s)/group(s)  |

For more information on OPC Server DCOM Security settings, please see the [DCOM Configuration for OPC Servers](d1b086e9-efc0-4185-9676-0d4d1398ade5.htm) chapter from the Deployment and Security section.

**OPC Client COM security settings**
|   Parameter    | Value      | 
| ------------- |:-------------:|
|Access Permissions area (both Edit Limits... and Edit Default... sections)      |- Add the domain/workgroup user(s) or group of users which have remote access permissions (in order for the local OPC Clients to receive callbacks from remote OPC Servers) <br> - Mark both Local Access and Remote Access rights for this (these) user(s)/group(s)  |

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/DCOMCNFG_MyComputer_ComSecurity.png)

For more information on OPC Client DCOM Security settings, please see the [DCOM Configuration for OPC Clients](82edbea3-8a73-4e82-942b-32cc9eb2111f.htm) chapter from the Deployment and Security section.

**OPC Server COM Identity**

In the left panel of the DCOMCNFG (Component Services window), further expand the **My Computer** node, then select the DCOM Config item. In the middle panel you will see

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/DCOMCNFG_Servers.png)

To be able to connect to the OPC Server with more than one OPC Client (local or remote), the OPC Server has to run under a preconfigured user identity (to avoid having each connecting user start its own OPC Server process). This identity may be different from the users added to the OPC Servers machine Launch and Activation Permissions and also from the users added to the OPC Server`s machine Access Permissions.

Yet, this identity has to be added to the OPC Client`s machine DCOM security settings, **Edit Limits...** and **Edit Default...** sections of the **Access Permissions** area.

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/DCOMCNFG_ServerIdentity.png)

### Note
```
New DCOM security settings will only take effect over DCOM applications once the applications are restarted.
```
### Note
```
DCOM security settings (Access Permissions and Launch and Activation Permissions) may be also
configured on a per-OPC Server basis. This covers the case where complex security layers are to be 
implemented.
```
DCOM Configuration

**More information on DCOM Security**

The OPC Servers constructed with Softing OPC Toolkit will have the DCOM security settings initialized with authentication on Connect and Identify impersonation level therefore only OPC Clients which run under a certain user account (account which has launch and activation permissions on the OPC Server`s machine) will be able to start the OPC Server. There is no point in using higher DCOM security settings since the DCOM package overhead will grow considerably, additional processing power will be required to encrypt/decrypt each call and these factors will drastically decrease the performance of the OPC Servers and Clients.

For a better data throughput, the OPC Clients constructed with Softing OPC Toolkit will have the DCOM security settings initialized with authentication set to None and Anonymous impersonation level. This is not a security hole since the OPC Clients may not be remotely started or accessed by other applications than the ones to which they hand their interfaces to. Technically only the OPC Servers to which the OPC Clients have succeeded in connecting and authenticating to may invoke callbacks to the respective OPC Clients. However, it is strongly advised to only grant rights to specific users and user groups.

Supposing that there are user accounts (groups) who may start and connect to the OPC Server (launch and activation permissions) and also have access to perform operations on the OPC Server (access permissions), there may be user accounts who only have the rights to connect to an already started OPC Server (activation permissions) and perform operations on the respective OPC Server (access permissions).

The OPC Client`s machine DCOM security settings will only suffer a minor modification, that is adding one or more users/groups to the global DCOM access permissions security settings. The added users (groups) should be the ones (should include the users) which are configured in the Identity field of the remote OPC Servers to which the OPC Client will connect to.

### Note
```
DCOM security is programmatically initialized in the Softing OPC Toolkit constructed OPC Server and/or 
OPC Client applications. If the provided security is not sufficient or is more than required, CoInitializeEx and 
CoInitializeSecurity calls may be used in the OPC Server and/or OPC Client applications with the required DCOM 
Security settings. The initialization inside the Softing OPC Toolkit Core (CoInitializeEx and CoInitializeSecurity) 
will fail with RPC_E_TOO_LATE (traces will only show result 0x01) but this is no problem as long as the process 
DCOM security has been correctly initialized. Another alternative to initialize the DCOM security in 
client/server applications is to use the provided API functionality to set the desired security levels, through the 
Application members DcomAuthLevel / DcomImpLevel for both clients and servers and DcomCbAuthLevel / 
DcomCbImpLevel for servers only.
```
DCOM Configuration

### Cross-domain DCOM security

In certain use-cases, the OPC DA/AE Clients and OPC DA/AE Servers may run on different machines which sometimes may rest in different domains.

It would not be possible to configure the same user with the same password in both domains, as the Windows domain users are identified by their unique ID. Instead, a local user must be created on all the machines that need to exchange OPC information (on both the server and client machines). The local user has to bear the same user name and must be assigned the same password on all machines.

Additional changes need to be made to the OPC DA Server running identity (the OPC DA Server must be running under the respective local user) and to the DCOM settings so that Local and Remote Launch, Activation and Access Permission must be granted.

On the OPC DA Client side, only the Access Permissions need to be modified so that the respective local user will be granted both Remote and Local Permissions.

### DCOM and Windows XP

When **DCOMCNFG** is executed, **Windows XP** shows a window for configuring the component services. To open the configuration dialog box for DCOM, select Component Services | Computers | My Computer and display the properties.

The default installation for XP forces remote users to authenticate as Guest. This means that DCOM clients cannot connect to a server running on an XP machine unless the Guest account is enabled and has enough rights to launch the server.

You can adjust these default settings from the control panel:

Administrative Tools | Local Security Policy | Local Policies | Security Options | Network access: Sharing and security model for local accounts

Change this setting to: "Classic - users authenticate as themselves".

### DCOM and Windows XP SP2, Windows Vista, Windows 7 and Windows 10

Starting with **Windows XP SP2**, the DCOM configuration distinguishes local and remote connections. Please ensure that the permissions are also assigned for remote access.

**Windows XP SP2** also allows configuring launch and access restrictions for the launch and access permissions. By default, remote access is restricted for the users Everyone and ANONYMOUS LOGON.

Remove this restriction if you want to grant access to everyone. If you want only specific users within a domain/workgroup to have remote access add those specific users instead of Everyone

### DCOM and Firewalls
DCOM cannot be used across a firewall. You have to either disable the firewall to allow remote communication or allow TCP and UDP port 135 and the dynamic configured DCOM port range. You can configure the dynamic DCOM port range as follows:

First, open the **My Computer Properties** window. Select the **Default Protocols** tab, make sure the **Connection-oriented TCP/IP** protocol is selected in the **DCOM Protocols** list and click the **Properties** button.

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/DCOMCNFG_DefaultProtocols.png)

A new window will appear: **Properties for COM Internet Services**. Click the **Add** button in order to add a port range (or a list of ports) to be used by the DCOM clients and servers.

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/DCOMCNFG_DefaultProtocols_Properties.png)

In the new window - **Add Port Range** - the desired port or port range can be provided. You can add as many ports or port ranges as required by your application (with respect to the number of clients/servers connected).

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/DCOMCNFG_DefaultProtocols_Properties_AddPortRange.png)

### Note
```
Windows/Network Firewalls will have to be configured to allow inbound traffic on port 135 (standard DCOM port 
used for establishing client-server communication) and also on all the ports configured in Default Protocols
(used for relying the successfully established DCOM connections).
```


### Server (32- and 64-bit) DCOM registration

The Softing OPC Classic Toolkit 4.45 provides support for both 32- and 64- bit compilation. For optimal functionality of a OPC Server this must be registered under a ProgID/ClsID and one runtime platform (32- or 64-bit). If switching from one Target Platform to another (32- to/from 64-bit) first the server instance must be unregistered (old platform), then the new server instance (new platform) must be registered.

### Obsolete/insecure DCOM configuration settings (no longer supported)
If the above DCOM configuring scenario seems rather more complex than desired, it is also possible to configure DCOM access in a more insecure manner. It is strongly recommended to only configure DCOM in this insecure manner if there is no other possibility of connecting the OPC DA/AE Server/Client Applications, as this represents a great security risk.
### Note
```
The insecure DCOM configuration is considered obsolete and is no longer supported, but it is provided due to 
convenience reasons.
```
Compared to the above settings, the following changes must be performed:

-   The **Default Authentication Level** and **Default Impersonation Level** security settings under **Default Properties** in the DCOM configuration window of **My Computer Properties** should be set to **None** and respectively **Impersonate**.
-   
![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/Obsolete_DCOMCNFG_DefaultProperties.png)

-   In the **Access Permissions** section, **Edit Limits...** button, make sure the users **ANONYMOUS LOGON**, **Everyone**, **INTERACTIVE**, **NETWORK** and **SYSTEM** have both **Local Access** and **Remote Access** permissions.
    
-   In the **Access Permissions** section, **Edit Defaults...** button, make sure the users **Everyone**, **INTERACTIVE**, **NETWORK** and **SYSTEM** have both **Local Access** and **Remote Access** permissions.
    
-   In the **Launch and Activation Permission** section, **Edit Limits...** button, make sure the users **Everyone**, **INTERACTIVE**, **NETWORK** and **SYSTEM** have the **Local Launch**, **Remote Launch**, **Local Activation** and **Remote Activation** permissions.
    
-   In the **Launch and Activation Permission** section, **Edit Defaults...** button, make sure the users **Everyone**, **INTERACTIVE**, **NETWORK** and **SYSTEM** have the **Local Launch**, **Remote Launch**, **Local Activation** and **Remote Activation** permissions.
    
-   For OPC Server machines it is required that the OpcEnum DCOM application and the registered OPC Servers under **Consoler Root** - **Component Services** - **Computers** - **My Computer** - **DCOM Config** have the **Authentication Level** set to **None** in the General Tab (of the properties window which can be accessed by selecting the respective DCOM service, right-clicking it and choosing Properties from the contextual menu).

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/Obsolete_OpcEnum.png)

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/Obsolete_GeneralServerProperties.png)

 Moreover, you will have to make sure that both the **Launch and Activation Permissions** and the **Access Permissions** are set to **Use Default** and also that the **Configuration Permissions** are set to **Customize** and the following users have both the **Full Control** and **Read** rights (open the **Change Configuration Permission** window by clicking the **Edit...** button.

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/Obsolete_SecurityServerProperties.png)

 In the **Identity** tab, either **The interactive user** (for OPC Servers running as Windows Applications) or **This user** (for OPC Servers running as Windows Services) must be selected, and the user logon details must be filled in where needed.
 
![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/Obsolete_ServerIdentityServerProperties.png)

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/Obsolete_ServiceIdentityServerProperties.png)

For Windows XP pre-SP2 systems, the **Local Security Policy** (**Control Panel** - **Administrative Tools** - **Local Security Policy**, **Security Settings** - **Local Policies** - **Security Options**) must have the following policies configured:
    
   -   **Network access: Let Everyone permission apply to anonymous users**: Enabled
        
    -   **Network access: Sharing and security model for local accounts**: Classic - local users authenticate as themselves

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/Obsolete_LocalSecurityPolicy.png)

-   For Windows XP SP2 and later, the DCOM configuration distinguishes between local and remote connections. Please ensure that the permissions are also assigned for remote access. Windows XP SP2 and later has enforces restrictions for the remote **Access Permissions** and **Launch and Activation Permissions**. By default, the user **Everyone** has no remote rights. Cancel this restriction if remote access needs to be granted to everyone.
