## **Update OPC Toolbox V4.1x Projects to OPC Toolkit V4.40**


### Summary

This article gives a brief description of the differences between the Softing OPC Toolbox V4.10 and Softing OPC Toolkit V4.40. Moreover it gives some hints to the code changes you should make to migrate your developed OPC application to the Softing OPC Toolkit V4.40.

Because the API of the OPC Toolkit (V4.40) has been changed due to compatibility issues with OPC Unified Architecture and due to user handling issues you will have to change the source code of your OPC Toolbox projects. The OPC Toolkit V4.40 (.NET) part has also been changed. In the new version we provide a .NET API to access the Toolkit Core API directly. Because of that matter you will have to rework your .NET OPC application as well if it was developed with the OPC Toolbox V4.10 (.NET).

###  OPC Servers

You can find an overview of the most important server interface classes which have to be used for server implementation at the [server concept](0c396293-a224-46ba-94dd-e821af146d72.htm) description.

When developing OPC Servers with the OPC Toolkit V4.40, basically you will have to change the main tasks which are:

-   Configuration of the OPC Server
    
-   Interfacing the Data Access I/O
    
-   Selection of the Application Type
    

###  Configuration of the OPC Server
Update OPC Toolbox V4.1x Projects to OPC Toolkit V4.40

The first difference between OPC Toolkit V4.40 and V4.10 is, that DA Namespace, AE area space and AE event space of an OPC server has been combined into a common Address Space.

The Address Space can include one **DaAddressSpaceRoot** or one **AeAddressSpaceRoot**, many **DaAddressSpaceElements** or **AeAddressSpaceElements**

In this connection nodes and leafs from V4.10 changed to **AddressSpaceElements** with and without children.

This means that you have to take this into account at address space definition/creation/parsing (e.g. at buildObjectSpaces()) for different address space types (e.g. [object based or string based address space](03cf9124-11f8-49e6-bcf9-3428c258b01e.htm))

If you would like to implement a hierarchy, you will have to add the address space elements accordingly to their hierarchical order.
### C
```
node = new MyDaAddressSpaceElement();
node.Name = "maths";
node.Type = MyDaAddressSpaceElement.TYPE_NODEMATH;
node.IoMode = EnumIoMode.NONE;
node.HasChildren = true;
Application.Instance.DaAddressSpaceRoot.AddChild(node);

g_sin = new MyDaAddressSpaceElement();
g_sin.Name = "sin";
g_sin.Type = MyDaAddressSpaceElement.TYPE_SINE;
g_sin.AccessRights = EnumAccessRights.READABLE;
g_sin.Datatype =  typeof(double);
g_sin.IoMode = EnumIoMode.REPORT;
node.AddChild(g_sin);
```
### VB .NET
```
node = New MyDaAddressSpaceElement
node.Name = "maths"
node.Type = MyDaAddressSpaceElement.TYPE_NODEMATH
node.IoMode = EnumIoMode.NONE
node.HasChildren = True
Application.Instance.DaAddressSpaceRoot.AddChild(node)

g_sin = New MyDaAddressSpaceElement
g_sin.Name = "sin"
g_sin.Type = MyDaAddressSpaceElement.TYPE_SINE
g_sin.AccessRights = EnumAccessRights.READABLE
g_sin.Datatype = GetType(Double)
g_sin.IoMode = EnumIoMode.REPORT
node.AddChild(g_sin)
```
### C++
```
node = (MyDaAddressSpaceElement*)creator->createMyDaAddressSpaceElement();
aName = tstring(_T("maths"));
node->setName(aName);
node->setType(MyDaAddressSpaceElement::TYPE_NODEMATH);
node->setIoMode(EnumIoMode_NONE);
node->setHasChildren(TRUE);
daRoot->addChild(node);

m_sin = (MyDaAddressSpaceElement*)creator->createMyDaAddressSpaceElement();
aName = tstring(_T("sin"));
m_sin->setName(aName);
m_sin->setType(MyDaAddressSpaceElement::TYPE_SINE);
m_sin->setAccessRights(EnumAccessRights_READABLE);
m_sin->setDatatype(VT_R8);
m_sin->setIoMode(EnumIoMode_REPORT);
node->addChild(m_sin);
```

### Interfacing the Data Access I/O (OPC DA)

In the OPC Toolbox V4.20 you will have to choose the update mechanism for active items by selecting the **EnumIoMode**. There are two mechanisms supported:

-   Polling
    
-   Reporting
    

A more detailed description can be found [here](2e1facc5-6688-4183-ac2b-7cdfba1c7b4a.htm).

These settings were done in OPC Toolbox V4.10 by selecting the itemTagUpdateMode of an item. After choosing the **EnumIoMode** you will have to change the implementation of the corresponding Methods used by the mechanism you have selected. e.g. **HandleReadRequests()**, **HandleWriteRequests()** of a transaction or **QueryCacheValue** of address space element.

In the OPC Toolbox V4.10 you could also set a itemTagDeviceIOMode to specify the IO mode for the device access. The OPC Toolkit V4.40 is using direct device access. If you want to have a queued device access you will have to separate the OPC communication from the device IO access.

Basically there are two different possibilities how the handling of the OPC communication (handling the incoming Requests) and the device IO access can be implemented. If you have a fast IO or you are working on memory areas in the context of the running process, the OPC communication and the device IO access can be combined within the according methods. (e.g. **HandleReadRequests**, of the Transaction class like in the sample [\V43x\Windows\C++\samples\server\Console](75ae2e25-6977-4820-8ead-ded22ee6f3c3.htm#Console))

If you have a slow IO or a somehow timeconsuming device IO access it is more likely to separate the OPC communication from the device IO access by filling a queue and having a separate thread which is then making the device IO access, and completing the requests from the queue. (like in the sample [\V43x\Windows\C++\samples\server\Serial_IO](75ae2e25-6977-4820-8ead-ded22ee6f3c3.htm#Serial_IO))

###  Handling Alarms and Events (OPC AE)

The OPC Alarms & Events specification describes different types of events. You can have simple events, tracking related events and condition related events. After the Eventsources have been configured in [buildNamespace](dca520c4-fd52-40a1-a4d5-76e36a096a48.htm) and [buildEventCategories](dca520c4-fd52-40a1-a4d5-76e36a096a48.htm) methods, the events can be triggered by calling the according methods (e.g. **Application::fireEvents**, **Application::changeConditions** or **AEEvent::fire**) which depend on the event type. The [server console sample](75ae2e25-6977-4820-8ead-ded22ee6f3c3.htm#Console) shows the implementation of an OPC DA and AE server and the handling of Events.

###  Selection of the Application Type

In the Toolkit V4.40 you will have to select what kind of OPC server application you are going to implement. To do this you will have to set the **Application type property**.

The OPC Toolkit V4.40 supports two application types:

-   **EnumApplicationType_EXECUTABLE** for Out-Of-Process OPC servers
    
-   **EnumApplicationType_LIBRARY** for In-Process OPC servers

### C#
```
Application.Instance.AppType = EnumApplicationType.EXECUTABLE;
```
### VB .NET
```
Application.Instance.AppType = EnumApplicationType.EXECUTABLE;
```
### C++
```
getApp()->setAppType(EnumApplicationType_EXECUTABLE);
```

Furthermore you will have to implement the corresponding methods depending on the application type you have selected. (e.g. Service methods like OnStart and OnStop if you would like to implement an OPC server running as windows service or DLL methods like DLLMain if you want to implement an In-Process OPC server)

###  OPC Clients

You can find an overview of the most important client interface classes which have to be used for client implementation at the [client concept](07026f9b-26d1-4ff7-90ff-8745ad4b7499.htm) description.

The main tasks which you will be faced with will be handling the possible object state changes of the OPC objects you are using. This will be DA/AE Sessions, DA/AE Subscriptions and DA/AE items. Furthermore you will have to implement the correct callback methods, for the received callbacks.(e.g. **handleReadCompleted**, **handleWriteCompleted**, **handleDataChanged** for asynchronous DataChange callbacks or **handleAeEventsReceived** for incoming alarms and events.)

The next thing you will have to decide is, which kind of operations you are going to use. You can perform different tasks like [reading items](e0b47f54-7b1a-4cf9-b5db-a4475d6e6229.htm) in a [synchronous or asynchronous](bc435923-131a-4d36-afa5-aa5ff28e3fab.htm) way. Synchronous operations can lower the performance of the communication, because they first return after the operation has been completed. You can also use asynchronous operations which return after their call immediatelly and then you receive e.g. the value changes or the object state changes in the according callback methods. (e.g. **handleDataChanged**)

Which kind or execution type is used is defined in the **executionType** property of the **executionOptions**. You can specify an **ExecutionType** which can be e.g. SYNCHRONOUS or ASYNCHRONOUS.

### C#
```
ExecutionOptions executionOptions = new ExecutionOptions();
executionOptions.ExecutionType = EnumExecutionType.ASYNCHRONOUS;
executionOptions.ExecutionContext = 0;
```
### VB .NET
```
Dim executionOptions As New ExecutionOptions()
executionOptions.ExecutionType = EnumExecutionType.ASYNCHRONOUS
executionOptions.ExecutionContext = Convert.ToUInt32(0)
```
### C++
```
ExecutionOptions executionOptions;
executionOptions.setExecutionType(ASYNCHRONOUS);
executionOptions.setExecutionContext(0);
```

Moreover for asynchronous operations you will have to increment the **ExecutionContext** to make sure that the received callbacks can be identified correctly.
### C#
```
if (ResultCode.FAILED(subscription.Read(100, itemList, out values, out results, executionOptions)))
{
    Console.WriteLine(" Subscription read failed!");
}
else
{
    executionOptions.ExecutionContext++;
}
```
### VB .NET
```
if ResultCode.FAILED (subscription.Read(Convert.ToUInt32(100), itemList, values, results, executionOptions)) then
    System.Console.WriteLine(" Subscription read failed")
Else
    Decimal.op_Increment(Convert.ToDecimal(executionOptions.ExecutionContext))
End If
```
### C++
```
if (FAILED (result = subscription->read(
                                    0,
                                    itemsToRead,
                                    valuesToRead,
                                    readResults,
                                    &executionOptions)))
{
    _tprintf(_T(" Subscription read failed "));
} //end if
else
{
    executionOptions.setExecutionContext(++executionContext);
} //end if ... else
```

The OPC Softing Toolkit implements a multithreaded architecture for OPC communication. This means that the communication to every single OPC server is handled in a separate thread. This ensures that possible application blocking scenarios are avoided. Your application is still working also if one of the currently running threads performs a timeconsumig synchronous operation or is in a blocking state.