## **Connection Monitor**

The Softing OPC Toolkit Client API offers a state-of-the-art Connection Monitoring mechanism. This mechanism allows you to get the most from the OPC Server with the OPC Clients built on top of the Softing OPC Toolkit.

The Connection Monitoring mechanism does not query the OPC Server by synchronously requesting the OCP Server`s status (each second), as this approach wastes valuable bandwidth and resources both on the server-side and on the client-side. Having that the DCOM security demands the OPC Client to authenticate per each call, there will also be a security overhead, but the performance loss arises from the fact that the server status response consists of a considerable collection of strings which has to be transported from the OPC Server side to the OPC Client side (copying, marshalling, OLE task memory operations). Rather, the Softing OPC Toolkit performs an interface query operation and interprets its result.

Once a connection problem is identified between the OPC Client and the OPC Server, the Client-side Toolkit Core disconnects all the client objects (notifying all the objects through the StateChanged callback/event and begins reconnecting to the OPC Server, phase 1. If the phase 1 reconnection has failed for the configured number of times, the pahse 2 of the reconnection process begins (this phase has an infinite number of retries). The Connection Monitor phase 2 only stops when the OPC Client application is terminated or when the Connection monitor is stopped (disabled).

### Note
```
The Softing OPC Toolkit Connection Monitor approach is the fastest possible connection monitor approach. A 
single DCOM timeout will occurr before disconnecting (and therefore notifying the Server Connection loss to) all 
the OPC Client-side Objects. No additional timeout will occurr when disconnecting these objects, thereby the OPC 
Client application will not freeze, lag or stop responding anyhow.
```

General usage of the Connection Monitor:
```
session.activateConnectionMonitor(true, 60000, 5, 10000, 30000);
```

The Connection Monitor has 5 parameters:

-   activate - whether to activate or not (to deactivate) the connection monitor;
-   checkCycle - interval in milliseconds at whcih to check the server connection;
-   connect1Attempts - number of attempts to retry the server connection in phase 1 (each "connect1Cycle" interval);
-   connect1Cycle - the interval in milliseconds at which to retry the server connection in phase 1 (will be used "connect1Attempts" times);
-   connect2Cycle - the interval in milliseconds at which to retry the server connection in phase 2 (will be used until client application is terminated or Connection Monitor deactivated).
### Note
```
The Client Objects have to be disconnected as once the OPC Server connection is reestablished, the OPC 
Handles corresponding to all the OPC Client Objects may be invalid (if the OPC Server was restarted or if the 
OPC Server also noticed the OPC Client disconnection and cleaned up the client objects).
```
Once the OPC Server connection has been reestablished, the Client-Side Toolkit Core recovers the previous OPC Client Object states (all the Client Objects which were Active will be activated, all the Client Objects which were Connected will be connected and the Client Objects which were Disconnected will remain disconnected.
### Note
```
It is recommended that the OPC Server check cycle to be greater than the DCOM operation timeout (30~45 
seconds).
```
