## **AE Condition List**

Every **AeSubscription** object keeps a list with all the active and/or not acknowledged condition-related events allocated to it. This list is maintained by interpreting the events received from an Alarms and Events server. The client can get the condition list using the **AeConditionList** property(.NET) and **getAeConditionList** method of an **AeSubscription** object(C++).
### Note
```
In C++ the above mentioned method returns an array of pointers to AeCondition objects. The objects in this array are 
destroyed internally in the toolkit, when the subscription destructor is called.
```
### Example

The following example shows how to receive notifications about the changes that appear in the condition list.

For a more detailed example on this topic see the AeCondition sample.
### C#
```
using Softing.OPCToolbox.Client;
using Softing.OPCToolbox;

...
// the method that handles the AeConditionsChanged event
private void HandleAeConditionsChanged(AeSubscription anAeSubscription, AeCondition[] conditions)
{
    // do some work with the list of conditions
}    

// gets the application instance
Application application = Application.Instance;

// initializes the application
application.Initialize();

// creates a new AeSession object and adds it to the application
AeSession session = new AeSession("opcae:///Softing.OPCToolboxDemo_ServerAE.1/{2E565243-B238-11D3-842D-0008C779D775}");

// creates a new AeSubscription object and adds it to the created session
AeSubscription subscription = new AeSubscription(session);    

// assigns a delegate to the AeConditionsChanged event             
subscription.AeConditionsChanged += new AeConditionsChangedEventHandler(HandleAeConditionsChanged);

// connects the session and subscription objects
session.Connect(true, true, new ExecutionOptions());                    
...

// disconnects session
session.Disconnect(new ExecutionOptions());

// removes subscription from session
session.RemoveAeSubscription(subscription);

// removes session from application
application.RemoveAeSession(session);

// terminates the application
application.Terminate();
...
```
### VB .NET
```
Imports Softing.OPCToolbox.Client
Imports Softing.OPCToolbox

...
'the method that handles the AeConditionsChanged event
Private Sub HandleAeConditionsChanged(ByVal anAeSubscription As AeSubscription, ByVal conditions As AeCondition())

    'do some work with the list of conditions
End Sub    

'gets the application instance
Dim application As Application = Application.Instance;

'initializes the application
application.Initialize()

'creates a new AeSession object and adds it to the application
Dim session As AeSession = new AeSession("opcae:///Softing.OPCToolboxDemo_ServerAE.1/{2E565243-B238-11D3-842D-0008C779D775}")

'creates a new AeSubscription object and adds it to the created session
Dim subscription As AeSubscription  = new AeSubscription(session)

'assigns a delegate to the AeConditionsChanged event             
AddHandler subscription.AeConditionsChanged, AddressOf HandleAeConditionsChanged

'connects the session and subscription objects
session.Connect(True, True, new ExecutionOptions())
...

'disconnects session
session.Disconnect(new ExecutionOptions())

'removes subscription from session
session.RemoveAeSubscription(subscription)

'removes session from application
application.RemoveAeSession(session)

'terminates the application
application.Terminate()
...
```
### C++
```
using namespace SoftingOPCToolboxClient;

class MyAeSubscription : public AeSubscription
{
    public:

        MyAeSubscription (AeSession* session) : AeSubscription(session) {}
        void handleAeConditionsChanged(const std::vector<AeCondition*>& someAeConditions)
        {
            // do some work with the list of conditions
        }    
};
...
int _tmain(int argc, _TCHAR* argv[])
{
    ExecutionOptions executionOptions;

    // gets an instance of the Application class
    Application* application = getApplication();

    // initializes the application
    application->initialize();

    // creates a new AeSession object and adds it to the application
    AeSession* session = new AeSession(_T("opcae:///Softing.OPCToolboxDemo_ServerAE.1/{2E565243-B238-11D3-842D-0008C779D775}"));

    // creates a new MyAeSubscription object and adds it to the created session
    MyAeSubscription* subscription = new MyAeSubscription(session);

    // connects the session and subscription objects; this determines conditionChanged events to be received;
    session->connect(TRUE, TRUE, &executionOptions);    
    ...

    // disconnect session and subscription
    session->disconnect(&executionOptions);

    // remove subscription
    session->removeAeSubscription(subscription);

    // remove session
    application->removeAeSession(session);

    // terminates the application
    application->terminate();    
    releaseApplication();

    delete session;
    delete subscription;
    ...
}
```
