## **Server**

You will find all the samples in the following directories:

###  Windows C++ samples:

<InstallationDir>\Softing\OPCToolkit\V44x\Windows\C++\samples\server

###  Windows .NET samples:

<InstallationDir>\Softing\OPCToolkit\V44x\Windows\NET\samples\server

Subdirectories with corresponding example names can be found. Provided server samples:

-   **[AddressSpace](75ae2e25-6977-4820-8ead-ded22ee6f3c3.htm#AddressSpace)** - console server application demonstrating the string-based namespace
-   **[Console](75ae2e25-6977-4820-8ead-ded22ee6f3c3.htm#Console)** – console server application demonstrating main OPC functionalities
-   **[Serial_IO](75ae2e25-6977-4820-8ead-ded22ee6f3c3.htm#Serial_IO)** – console server application demonstrating how to separate the OPC communication and the device communication
-   **[ServerDLL](75ae2e25-6977-4820-8ead-ded22ee6f3c3.htm#ServerDLL)** - console server application demonstrating how to use a server library generated by the Project Wizard
-   **[Session](75ae2e25-6977-4820-8ead-ded22ee6f3c3.htm#Session)** - console server application demonstrating session based access
-   **[UpdateMode](75ae2e25-6977-4820-8ead-ded22ee6f3c3.htm#UpdateMode)** - console server application demonstrating the use of different IO modes

###  AddressSpace

**Access types** supported by this example:

-   OPC Data Access Specification V1.0a, V2.05 and V3.0
-   OPC XML Data Access Specification V1.01

This example demonstrates the use of the OPC Toolkit to build/rebuild the object-based namespace (created by the server statically) using both random and not random values. The additionally implemented technique regards the string-based namespace (created at runtime by the client dynamically). If the string-based namespace follows a special syntax (e.g. STEP-7 syntax) it is also called syntax-based namespace. The object-based namespace object tree is created upon starting the server and the string-based can be dynamically added, item-by-item, by the client. The object-based items are periodically generated and the namespace is rebuilt accordingly. The current namespace is printed to the console window.

The string-based namespace functionality is achieved by implementing the createInternalAddressSpaceElement method of the creator:

MyCreator::createInternalDaAddressSpaceElement.

Once created, the namespace can be queried only if the following methods of the address space root are implemented:

MyDaAddressSpaceRoot::queryAddressSpaceElementData

MyDaAddressSpaceRoot::queryAddressSpaceElementChildren.

The handling of the requests is done by implementing the following methods of the transaction class:

MyTransaction::handleReadRequests

MyTransaction::handleWriteRequests.

All items have read/write access and input/output mode set as REPORT.

In order to get string-based item, this must be added to the namespace by right-clicking "group" node in the data access explorer on the right, choosing "Add item" from the pop-up menu and then typing the name of the item (for example "stringBased.N1.T1").

Both, object-based and string-based, items are not initialized with a value and their quality is set to "BAD (wainting for values)". Since they are marked with read/write access, a new value can be set using the Demo Client; for this, point to the desired item, enter the value in the textbox near to the upper-right corner and press the "Write" button.

The OPC XML Data Access support can be achieved easily by setting the IP, the port and the URL of the server; the following methods of the Application class must be executed:

Application::setIpPortHTTP

Application::setUrlDa

The server can be added to the Demo Client's server list by typing "http://localhost:8079/OPC/DA" in the URL textbox of the OPC Servers page. Then it can be browsed and items can be added like in the case of the OPC Data Access.

Please consult the codebase of the sample to familiarize with the functionalities described above!

The following image presents both the object-based and string-based namespace items.

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/addressspace.png)

### Console

**Access types** supported by this sample:

-   OPC Data Access Specification V1.0a, V2.05 and V3.0
-   OPC XML Data Access Specification V1.01
-   OPC Alarms and Events Specification Version 1.10

This sample demonstrates the use of the OPC Toolkit to build the string-based (created by the server statically) DA and AE namespace. The namespace object tree is created upon starting the server.

The sample also provides OPC XML-DA access. This can be achieved easily by setting the IP and the port and the URL of the server; the following functions of the Application must be executed:

setIpPortHTTP

setUrlDa

The server can be added to the Demo Client's server list by typing "http://localhost:8081/OPC/DA" in the URL textbox of the OPC Servers page.

For the present example also web based access is implemented, thus providing access from an internet browser (like Internet Explorer or Mozilla Firefox), at the "http://localhost:8081/index.html". This functionality was obtainded by extending the WebTemplate class and implementing the following method of the Creator:

createWebTemplate

The Application must also be prepared by using the following functions:

setWebRootDirectory

setWebRootFile

setWebAdministratorPassword

setWebOperatorPassword

### Note
```
The "www" directory of the console sample must be copied to the directory where the executable is located!
```

In order to get string-based namespace functionality the following Creator methods must be implemented:

createInternalDaAddressSpaceElement

createInternalAeAddressSpaceElement

Once created the namespace can be queried only if the following methods are implemented:

queryAddressSpaceElementData

queryAddressSpaceElementChildren

The handling of the requests is done by implementing the:

handleReadRequests

handleWriteRequests

methods.

All DA items have read/write access and input/output mode set as either POLL or REPORT. The AE namespace consist in the Area and Event space. AE events are generated at every second, minute and hour change. Two AE conditions are also generated in order to exemplify this functionality.

Please consult the codebase of the sample to familiarize with the functionalities described above!

The following image presents both, DA and AE string-based namespace.

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/console.png)

### Serial_IO

**Access types** supported by this sample:

-   OPC Data Access Specification V1.0a, V2.05 and V3.0
-   OPC XML Data Access Specification V1.01

This sample demonstrates how to separate the OPC communication from device communication. This is done by having two threads: one for the device communication and one for the OPC communication. The address space is created upon starting the server. It contains two AddressSpaceElements that have read/write access and input/output mode set as POLL. After creating the address space the simulation threads are created and started. They will generate random values for the simulation elements.

The sample also provides OPC XML-DA access. This can be achieved easily by setting the IP and the port and the URL of the server; the following functions of the Application must be executed:

setIpPortHTTP

setUrlDa

The server can be added to the Demo Client's server list typing "http://localhost:8079/OPC/DA" in the URL textbox of the OPC Servers page.

In order to get address space functionality the following Creator methods must be implemented:

createInternalDaAddressSpaceElement

Once created the namespace can be queried only if the following methods are implemented:

queryAddressSpaceElementData

The handling of the requests is done by implementing the:

handleReadRequests

handleWriteRequests

methods.

Please consult the codebase of the sample to familiarize with the functionalities described above!

The following image presents the way the two values are updated with randomly generated values in the Softing OPC Demo Client.

![OPC-Classic-](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/IOMechanism.png)


### ServerDLL

**Access types** supported by this sample:

-   OPC Data Access Specification V1.0a, V2.05 and V3.0
-   OPC XML Data Access Specification V1.01
-   OPC Alarms and Events Specification Version 1.10

This sample demonstrates how to consume a DLL that is the outcome of a Project Wizard generated project.

The **ServerDLL** folder contains two subfolders. First one, **OpcProject**, contains a server project generated using the wizard. The outcome of this project is a DLL called OpcPRoject.dll and is consumed in the second project, placed in the TestDLL folder.

The DLL exposes two methods: **Start** and **Stop**.

Calling the Start method will initialize the server application, will create the DA and the AE address spaces, will generate random values for the created DaAddressSpaceElement and will generate an AeEvent at every second.

Calling the Stop method will stop the value and event generation and will terminate the server application.

### Note
```
Before running the TestDll project, make sure that you have placed the OpcProject.dll in the folder of TestDll.exe.
```

Please consult the codebase of the sample to familiarize with the functionalities described above!

###  Session

**Access types** supported by this sample:

-   OPC Data Access Specification V1.0a, V2.05 and V3.0
-   OPC XML Data Access Specification V1.01

This example demonstrates the use of the OPC Toolkit to build a namespace (created by the server statically) and to identify the session type of the connected clients. Also access based on login identification is implemented. The namespace object tree is created upon starting the server and contains two items, one with READ/WRITE and one with READ access. The value of the item can be written only if the client session is a secured one - the user needs to be logged in to the server with the right user ("OPC") and password ("opc"). The input/ouput mode is set to POLL and POLL_OWNCACHE. The current namespace, the connected clients and their session types are printed to the console window.

This example also provides OPC XML-DA access. This can be achieved easily by setting the IP and the port and the URL of the server; the following functions of the Application must be run:

setIpPortHTTP

setUrlDa

The server can be added to the Demo Client's server list by typing "http://localhost:8079/OPC/DA" in the URL textbox of the OPC Servers page.

The identification of the sessions can be achieved by extending the DaSession class

The handling of the requests is done by implementing the:

handleReadRequests

handleWriteRequests

methods.

All items have read/write access and input/output mode set as POLL and POLL_OWNCACHE.

Please consult the codebase of the sample to familiarize with the functionalities presented above!

The following image presents the namespace items both on DA and XML-DA server. The console server application window also displays the information regarding the session types of the connected clients and the items of the namespace:

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/session.png)


### UpdateMode

**Access types** supported by this sample:

-   OPC Data Access Specification V1.0a, V2.05 and V3.0
-   OPC XML Data Access Specification V1.01

This example shows how to change the items values when the items have different IO mode. For this, the server address space is built to contain two types of AddressSpaceElements: ones that have the IO mode set to EnumIoMode_REPORT_CYCLIC and others that have IO mode set to EnumIoMode_POLL. For the first category a new method was created (**changeValue**) to change their values. For the second category, the **handleReadRequests** method is used to change the values. The address space is created upon starting the server.

The sample also provides OPC XML-DA access. This can be achieved easily by setting the IP and the port and the URL of the server; the following functions of the Application must be executed:

setIpPortHTTP

setUrlDa

The server can be added to the Demo Client's server list by typing "http://localhost:8079/OPC/DA" in the URL textbox of the OPC Servers page.

In order to have address space functionality the following Creator methods must be implemented:

createInternalDaAddressSpaceElement

Once created the address space can be queried only if the following methods are implemented:

queryAddressSpaceElementData

The handling of the requests is done by implementing the:

handleReadRequests

method.

Please consult the codebase of the sample to familiarize with the functionalities described above!

The following image presents how the items values are generated and displayed in the Softing OPC Demo Client.

![OPC-Classic-SDK](https://github.com/SoftingIndustrial/OPC-Classic-SDK/raw/main/documentation_pics/UpdateMode.png)