## **DCOM Configuration for OPC Servers**

### Note
```
Administrative priviledges are required to perform the following Windows DCOM Security configuration.
```
If the deployed OPC Application is an OPC DA/AE Server, the Windows DCOM Security must be configured to only grant allowed users the permissions to remotely start and remotely connect to the OPC Servers the machine exposes.

It is reccommended that the DCOM Security settings to be configured in a very secured way in order to avoid unauthorized access.

### Note
```
There is no need in configuring the Windows DCOM Securty settings for local OPC DA/AE Clients connecting to local 
OPC DA/AE Servers. Also, pure OPC XML-DA Clients/Servers do not require DCOM configuration.
```

The "OPC DA/AE Client" will be further referred to as "OPC Client" and the "OPC DA/AE Server" will be further referred to as "OPC Server". Also, the "Domain/Workgroup user" will be further referred to as "user".

The Windows DCOM Security settings to be configured for an OPC Client machine are **Access Permissions**, **Launch and Activation Permissions** and **Identity**. These sections decide "as which user" the remote OPC Client has the right to start and connect to the local OPC Server, respectively under which user the OPC Server runs as.

### Note
```
It is recommended for the OPC Server to have a running user configured in its identity tab. Not setting a predefined 
identity to the OPC Server would cause it to be launched several times for each OPC Client running under a user 
with both launch and activation permissions. This is not the designed behavior of an OPC Server.
```

The OPC Server`s security settings may be configured to allow two different levels of access to remote OPC Clients based on the users these are running under:

-   Remote start, remote activation and remote access: granting a user this access would allow OPC Clients running under this user account to remotely launch, remotely connect to and remotely exchange data with an OPC Server;
-   Remote activation and remote access: granting a user this access would allow OPC Clients running under this user account to remotely connect to and remotely exchange data with an OPC Server. This will not allow the respective OPC Clients to remotely start the OPC Server (launch the process), if it is not running.

The following steps must be followed to properly configure the Windows DCOM Security settings on a machine where an OPC Server is to be deployed:

-   Open the **Component Services** addin from the Control Panel or simply run **DCOMCNFG.EXE**;
-   Expand the (left panel)**Console Root**, expand the **Component Services**, expand the **Computer** and then select **Properties** from the **My Computer** contextual menu;
-   In the newly opened **My Computer Properties** window, select the **Default Properties** tab and make sure the following settings are configured (default Windows settings):
    -   **Enable Distributed COM on this computer** - checked
    -   **Default Authentication Level** - Connect
    -   **Default Impersonation Level** - Identify
-   Select the **COM Security** tab and perform the following operations:
    -   Click the **Edit Limits...** button of the **Access Permissions** controls group and in the newly opened **Access Permissions** window add the required users with both **Local Access** and **Remote Access** rights then press OK;
    -   Click the **Edit Default...** button of the **Access Permissions** controls group and in the newly opened **Access Permissions** window add the required users with both **Local Access** and **Remote Access** rights then press OK;
### Note
```
Users from both access levels (launch and access-only rights) must be added to both of the two lists 
above.
```



-   -   Click the **Edit Limits...** button of the **Launch and Activation Permissions** controls group and in the newly opened **Launch and Activation Permissions** window add the required users with **Local Launch**, **Remote Launch**, **Local Activation** and **Remote Activation** rights then press OK;
    -   Click the **Edit Default...** button of the **Launch and Activation Permissions** controls group and in the newly opened **Launch and Activation Permissions** window add the required users with **Local Launch**, **Remote Launch**, **Local Activation** and **Remote Activation** rights then press OK;

```
Please only grant **Local Launch** and **Remote Launch** rights to the highest access level (users who 
have the right to launch an OPC Server) in both of the two lists above.
```
-   Close the **My Computer Properties** window by pressing OK;
-   Expand the **My Computer** node on the left panel of the **Component Services** window and select the **DCOM Config** node;
-   Search for the version-independent Program ID and select **Properties** from its contextual menu;
-   Navigate to the identity tab, select **This user** and input the credentials for the user under which the OPC Server will run under;
-   Per-OPC Server Access Permissions and Launch and Activation Permissions may be configured under the Security tab of this window;
### Note
```
The additional access permissions and Launch and activation permissions may be configured "per individual 
OPCServer" but these settings will be lost if the OPC Server is unregistered from the system.
```

For more information on General Windows DCOM Security settings please see the [DCOM Configuration](6674f8ee-0d1c-484c-afdb-26d7231e089e.htm) chapter from the Installation section.

The most secured way of configuring DCOM Security settings on a Windows machine where an OPC Server runs is to only grant remote access permissions and remote launch and activation permissions to the users the remote OPC Clients run under, users that should have the rights to also start the OPC Server if it is not already running when connecting to it. Only grant remote access permissions and remote activation permissions to the users the remote OPC Clients run under, users that should not have the rights to start the OPC Server if it is not already running, but only to connect to it and perform varions OPC-specific operations. Each action of each user may be audited.

The most insecured way of configuring Windows DCOM Security settings where an OPC Client runs is to grant remote access permissions and remote launch and activation permissions to the special "ANONYMOUS LOGON", "Everyone", "Network" and "SYSTEM" Windows Users Accounts. This is the most unsecure Windows DCOM Security configuration and should be never used as it allows unsecured remote access to anybody.